<?php
/**
 * @package	HikaShop for Joomla!
 * @version	4.7.0
 * @author	hikashop.com
 * @copyright	(C) 2010-2023 HIKARI SOFTWARE. All rights reserved.
 * @license	GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined('_JEXEC') or die('Restricted access');

use Baikalsr\Lib\Tools;
use Joomla\CMS\Layout\FileLayout;
use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;

JLoader::register('Baikalsr\Lib\Tools', dirname(__FILE__) . '/lib/tools.php');

class plgHikashopshippingBaikalsr extends hikashopShippingPlugin {
	var $baikalsr_methods = array(
		array('key' => '1', 'code'=>'courier_shipping', 'name' => 'PLG_HIKASHOPSHIPPING_COURIER_SHIPPING', 'countries' => 'RUSSIA'),
		array('key' => '2', 'code'=>'pickup_shipping', 'name' => 'PLG_HIKASHOPSHIPPING_PICKUP_SHIPPING', 'countries' => 'RUSSIA')
	);

	var $convertUnit=array(
		'kg' => 'KGS',
		'lb' => 'LBS',
		'cm' => 'CM',
		'in' => 'IN',
		'kg2' => 'kg',
		'lb2' => 'lb',
		'cm2' => 'cm',
		'in2' => 'in',
	);
	
	protected $autoloadLanguage = true;
	
	public $nbpackage = 0;
	public $package_added = 0;

	var $multiple = true;
	var $name = 'baikalsr';
	
	function __construct(&$subject, $config) {
		parent::__construct ($subject, $config);
		$this->baikalsrTools = new Tools();
	}
	
	private function init() {
		static $init = null;
		if($init !== null)
			return $init;	
		
		JHtml::_('script', 'jui/jquery.autocomplete.min.js', array('version' => 'auto', 'relative' => true));
		JHtml::_('script', 'system/validate.js', array('version' => 'auto', 'relative' => true));
		$doc = JFactory::getDocument();
		$base = (hikashop_isClient('administrator')) ? '..' : JURI::base(true);
		$doc->addScriptDeclaration('var translate = {
			no_data: "' . JText::_( 'PLG_HIKASHOPSHIPPING_NO_DATA') . '",
			send_request: "' . JText::_('PLG_HIKASHOPSHIPPING_SEND_REQUEST') . '",
			entry_fio: "' . JText::_( 'PLG_HIKASHOPSHIPPING_FIO') . '",
			entry_inn_kpp: "' . JText::_( 'PLG_HIKASHOPSHIPPING_INN_KPP') . '",
			entry_inn: "' . JText::_( 'PLG_HIKASHOPSHIPPING_INN') . '",
			err_title: "' . JText::_( 'ERROR') . '",
			warning_title: "' . JText::_('WARNING') . '",
			entry_contact_person: "' . JText::_( 'PLG_HIKASHOPSHIPPING_CONTACT_PERSON') . '",
			min_term: "' . JText::_( 'PLG_HIKASHOPSHIPPING_TERM_LBL2') . '", 
			shipping_cost: "' . JText::_( 'PLG_HIKASHOPSHIPPING_SHIPPING_COST') . '", 
			err_shipping_cost: "' . JText::_( 'PLG_HIKASHOPSHIPPING_ERROR_SHIPPING_COST') . '",
		};');
		$doc->addScript($base.'/plugins/hikashopshipping/' . $this->name . '/assets/js/' . $this->name . '.js');
		$doc->addStyleSheet($base.'/plugins/hikashopshipping/' . $this->name . '/assets/css/' . $this->name . '.css');	
	
		$init = true;
		return $init;
	}
	
	
	function shippingMethods(&$main) {
		$methods = array();
		foreach ($this->baikalsr_methods as $method) {
			$methods[$main->shipping_id . '-' . $method['key']] = JText::_($method['name']);
		}
		return $methods;
	}
	
	/**
	 * Этот метод будет вызван HikaShop на странице настройки метода доставки.
	 *
	 * @param object $element Метод доставки
	*/	
	function onShippingConfiguration(&$element) {
		$subtask = hikaInput::get()->getCmd('subtask', '');		
		if($subtask != 'shipping_edit' && $subtask != 'edit')
			return $this->plgHikaOnSelfCallBE($subtask);
		
		if(!$this->init())
			return '';
		$config = &hikashop_config();
		$this -> main_currency = $config -> get('main_currency', 1);
		$currencyClass = hikashop_get('class.currency');
		$currency = hikashop_get('class.currency');
		$this -> currencyCode = $currency -> get($this -> main_currency)->currency_code;
		$this -> currencySymbol = $currency -> get($this -> main_currency)->currency_symbol;

		$this -> baikalsr = hikaInput::get()->getCmd('name', 'baikalsr');
		$this -> categoryType = hikashop_get('type.categorysub');
		$this -> categoryType -> type = 'tax';
		$this -> categoryType -> field = 'category_id';
		
		parent::onShippingConfiguration($element);
			
	}
	
	/**
	 * Этот метод будет вызван HikaShop в серверной части при сохранении конфигурации метода доставки.
	 *
	 * @param object $element Метод доставки
	*/
	function onShippingConfigurationSave(&$element) {
		return true;
	}
	
	/**
	 * Метод установки настроек метода доставки по умолчанию.
	 *
	 * @param object $element Метод доставки
	*/	
	function getShippingDefaultValues(&$element){
		$element -> shipping_name = 'Байкал-сервис ТК';
		$element -> shipping_description = '';
		$element -> group_package = 0;
		$element -> shipping_images = 'baikalsr';
		$element -> shipping_params -> post_code = '';
		$element -> shipping_currency_id = $this -> main_currency;
	}
	
	function _getShippingPopupData($order_id, $post_data = array()) {			
		
		$orderClass = hikashop_get('class.order');
		$order = $orderClass->loadFullOrder($order_id);
		if(empty($order)){
			return JFactory::getApplication()->enqueueMessage(JText::sprintf('PLG_HIKASHOPSHIPPING_NO_ENTRY_FOUND', $order_id), 'error');
		}
				
		//get shipment config
		$shippingClass = hikashop_get('class.shipping');
		$config = $shippingClass->get($order->order_shipping_id);
		//Установим ключ api для выполнения расчета
		if (isset($config->shipping_params->test) && isset($config->shipping_params->authKeyTest) && isset($config->shipping_params->authKey)){
			if($config->shipping_params->test){
				$this->baikalsrTools::defineAuthParams( $config->shipping_params->authKeyTest, true );
			}else{
				$this->baikalsrTools::defineAuthParams( $config->shipping_params->authKey, false );
			}				
		}else{
			return JFactory::getApplication()->enqueueMessage(JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_KEY'), 'error');
		}
		$this->form_data = new stdClass();		
		//get shipment order by order_id
		$db = JFactory::getDBO();
		$query = 'SELECT * FROM ' . hikashop_table('plg_baikalsr_shipping') . ' WHERE order_id = ' . $order_id. ' LIMIT 1';
		$db -> setQuery($query);
		$shipment = $db->loadObject();
		if(empty($shipment))
			$shipment = new stdClass();
		
		if(isset($shipment->shipment_data) && !empty($shipment->shipment_data)){
			$this->disabled = true;			
			$sent_data = json_decode($shipment->shipment_data, true);
		}elseif(!empty($post_data)){
			//$sent_data = $this->_prepareData($post_data);
			$sent_data = $post_data;
			$this->warning = $this->_validateForm($sent_data);
			if(count((array)$this->warning) == 0){
				$baikalResponse = $this->baikalsrTools::order(array_merge($sent_data, ['order_id' => $order_id]));
				$this->errors = new stdClass();
				if(empty($baikalResponse)) {
					$this->errors->no_response = JText::_('PLG_HIKASHOPSHIPPING_ERROR_API_NO_RESPONSE');
				}elseif(isset($baikalResponse["errors"]) && !empty($baikalResponse["errors"])) {
					//$this->errors->invalid_data = implode("</li><li>", $baikalResponse["errors"]);
					foreach($baikalResponse["errors"] as $errorData)
						$this->errors->invalid_data .= urldecode(str_replace("=", ": ", http_build_query($errorData,"", "\r\n")));							
				}elseif(!isset($baikalResponse["request_hash"]) || empty($baikalResponse["request_hash"])){
					$this->errors->no_hash = JText::_('PLG_HIKASHOPSHIPPING_ERROR_API_NO_HASH');
				}elseif(!isset($baikalResponse["request_id"]) || empty($baikalResponse["request_id"])){
					$this->errors->no_request = JText::_('PLG_HIKASHOPSHIPPING_ERROR_API_NO_REQUEST');
				}elseif(!isset($baikalResponse["tracknumbers"]) || empty($baikalResponse["tracknumbers"])){
					$this->errors->no_tracking = JText::_('PLG_HIKASHOPSHIPPING_ERROR_API_NO_TRACKING');
				}
				if(count((array)$this->errors) == 0){
					$shipment->id = isset($shipment->id) ? $shipment->id : NULL;
					$shipment->order_id = $order_id;
					$shipment->shipmentmethod_id = $config->shipping_id;
					$shipment->request_id = $baikalResponse["request_id"];
					$shipment->order_number = $order->order_number;
					$shipment->tracking_number = $baikalResponse["tracknumbers"][0];
					$shipment->shipment_data = $sent_data;
					$shipment->status_name = JText::_('PLG_HIKASHOPSHIPPING_STATUS_0'); 
					$shipment->shipment_cost = $order->order_shipping_price;
					$shipment->shipment_actual_cost	= NULL;
					$shipment->shipment_add_cost = NULL;
					$date   = Factory::getDate();
					$shipment->date_added = (string) $date;
					$this->_updateShipmentOrder($shipment);							
					$this->disabled = true;	
					$this->success = true;						
				}
			}
		}

		if(isset($this->success) && $this->success){
			$this->shipping_html = $this->_getShippingInfoBody($order);
		}
		
		if(isset($shipment->request_id) && !empty($shipment->request_id)){	
			$this->form_data->request_id = $shipment->request_id;
		}
		if(isset($shipment->tracking_number) && !empty($shipment->tracking_number)){	
			$this->form_data->tracking_number = $shipment->tracking_number;
		}
		$this->form_data->order_id = $order_id;
		$this->form_data->shipmentmethod_id = $order->order_shipping_id;
		
		//API
		$this->form_data->authKey = $config->shipping_params->authKey;
		$this->form_data->authKeyTest = $config->shipping_params->authKeyTest;
		$this->form_data->test_mode = $config->shipping_params->test;
		
		//Consignor
		if (isset($sent_data['consignor_contragent_id'])) {
			$this->form_data->consignor_contragent_id = $sent_data['consignor_contragent_id'];
		} else {
			$this->form_data->consignor_contragent_id = $config->shipping_params->consignor_contragent_id;
		}	
		if (isset($sent_data['author_lastname'])) {
			$this->form_data->author_lastname = $sent_data['author_lastname'];
		} else {
			$this->form_data->author_lastname = $config->shipping_params->author_lastname;
		}	
		if (isset($sent_data['author_firstname'])) {
			$this->form_data->author_firstname = $sent_data['author_firstname'];
		} else {
			$this->form_data->author_firstname = $config->shipping_params->author_firstname;
		}	
		if (isset($sent_data['author_middlename'])) {
			$this->form_data->author_middlename = $sent_data['author_middlename'];
		} else {
			$this->form_data->author_middlename = $config->shipping_params->author_middlename;
		}
		if (isset($sent_data['consignor_fio'])) {
			$this->form_data->consignor_fio = $sent_data['consignor_fio'];
		} else {
			$this->form_data->consignor_fio = $config->shipping_params->author_lastname.' '.$config->shipping_params->author_firstname.' '.$config->shipping_params->author_middlename;
		}						
		if (isset($sent_data['consignor_phone'])) {
			$this->form_data->consignor_phone = $sent_data['consignor_phone'];
		} else {
			$this->form_data->consignor_phone = $config->shipping_params->consignor_phone;
		}		
		if (isset($sent_data['consignor_email'])) {
			$this->form_data->consignor_email = $sent_data['consignor_email'];
		} else {
			$this->form_data->consignor_email = $config->shipping_params->consignor_email;
		}
		//Consignee
		if (isset($sent_data['consignee_company'])) {
			$this->form_data->consignee_company = $sent_data['consignee_company'];
		} else {
			$this->form_data->consignee_company = '';
			if(isset($order->billing_address->address_company) && !empty($order->billing_address->address_company))
				$this->form_data->consignee_company = $order->billing_address->address_company;
			if(isset($order->shipping_address->address_company) && !empty($order->shipping_address->address_company))
				$this->form_data->consignee_company = $order->shipping_address->address_company;			
		}
		$this->form_data->consignee_company = htmlspecialchars_decode($this->form_data->consignee_company);
		if (isset($sent_data['consignee_physical_person'])) {
			$this->form_data->consignee_physical_person = $sent_data['consignee_physical_person'];
		} else {
			if (isset($this->form_data->consignee_company) && !empty($this->form_data->consignee_company)) {
				$this->form_data->consignee_physical_person = 0;
			}else{
				$this->form_data->consignee_physical_person = 1;
			}
		}
		if (isset($sent_data['consignee_inn'])) {
			$this->form_data->consignee_inn = $sent_data['consignee_inn'];
		} else {
			$this->form_data->consignee_inn = '';
			if(isset($order->billing_address->inn) && !empty($order->billing_address->inn))
				$this->form_data->consignee_inn = $order->billing_address->inn;
			if(isset($order->shipping_address->inn) && !empty($order->shipping_address->inn))
				$this->form_data->consignee_inn = $order->shipping_address->inn;		
		}	
		if (isset($sent_data['consignee_kpp'])) {
			$this->form_data->consignee_kpp = $sent_data['consignee_kpp'];
		} else {
			$this->form_data->consignee_kpp = '';
			if(isset($order->billing_address->kpp) && !empty($order->billing_address->kpp))
				$this->form_data->consignee_kpp = $order->billing_address->kpp;
			if(isset($order->shipping_address->kpp) && !empty($order->shipping_address->kpp))
				$this->form_data->consignee_kpp = $order->shipping_address->kpp;		
		}
		if (isset($sent_data['consignee_inn_kpp'])) {
			$this->form_data->consignee_inn_kpp = $sent_data['consignee_inn_kpp'];
		} else {
			$this->form_data->consignee_inn_kpp = trim($this->form_data->consignee_inn . ' ' . $this->form_data->consignee_kpp);
			$pos = strpos($this->form_data->consignee_inn_kpp, ' ');
			if ($pos !== false) {
				$this->form_data->consignee_inn_kpp = substr_replace($this->form_data->consignee_inn_kpp, '/', $pos, strlen(' '));
			}
		}	
		if (isset($sent_data['consignee_fio'])) {
			$this->form_data->consignee_fio = $sent_data['consignee_fio'];
		} else {
			$this->form_data->consignee_fio = '';				
			if(isset($order->shipping_address->address_lastname) && !empty($order->shipping_address->address_lastname))
				$this->form_data->consignee_fio .= $order->shipping_address->address_lastname;
			if(isset($order->shipping_address->address_firstname) && !empty($order->shipping_address->address_firstname))
				$this->form_data->consignee_fio .= ' ' . $order->shipping_address->address_firstname;
			if(isset($order->shipping_address->address_middle_name) && !empty($order->shipping_address->address_middle_name))
				$this->form_data->consignee_fio .= ' ' . $order->shipping_address->address_middle_name;
			$this->form_data->consignee_fio = trim($this->form_data->consignee_fio);
			if(isset($this->form_data->consignee_fio) && empty($this->form_data->consignee_fio)){
				if(isset($order->billing_address->address_lastname) && !empty($order->billing_address->address_lastname))
					$this->form_data->consignee_fio .= $order->billing_address->address_lastname;
				if(isset($order->billing_address->address_firstname) && !empty($order->billing_address->address_firstname))
					$this->form_data->consignee_fio .= ' ' . $order->billing_address->address_firstname;
				if(isset($order->billing_address->address_middle_name) && !empty($order->billing_address->address_middle_name))
					$this->form_data->consignee_fio .= ' ' . $order->billing_address->address_middle_name;
				$this->form_data->consignee_fio = trim($this->form_data->consignee_fio);	
			}
		}
		if (isset($sent_data['consignee_legal_opf_id'])) {
			$this->form_data->consignee_legal_opf_id = $sent_data['consignee_legal_opf_id'];
		} else {
			$this->form_data->consignee_legal_opf_id = 4;
		}	
		if (isset($sent_data['consignee_physical_document_id'])) {
			$this->form_data->consignee_physical_document_id = $sent_data['consignee_physical_document_id'];
		} else {
			$this->form_data->consignee_physical_document_id = 1;
		}
		if (!isset($sent_data['consignee_no_document'])) {
			$this->form_data->consignee_no_document = false;
		} else {
			$this->form_data->consignee_no_document = $sent_data['consignee_no_document'];
		}
		if (isset($sent_data['consignee_seria'])) {
			$this->form_data->consignee_seria = $sent_data['consignee_seria'];
		} else {
			$this->form_data->consignee_seria = '';
			if(isset($order->billing_address->seria) && !empty($order->billing_address->seria))
				$this->form_data->consignee_seria = $order->billing_address->seria;
			if(isset($order->shipping_address->seria) && !empty($order->shipping_address->seria))
				$this->form_data->consignee_seria = $order->shipping_address->seria;		
		}	
		if (isset($sent_data['consignee_number'])) {
			$this->form_data->consignee_number = $sent_data['consignee_number'];
		} else {
			$this->form_data->consignee_number = '';
			if(isset($order->billing_address->number) && !empty($order->billing_address->number))
				$this->form_data->consignee_number = $order->billing_address->number;
			if(isset($order->shipping_address->number) && !empty($order->shipping_address->number))
				$this->form_data->consignee_number = $order->shipping_address->number;		
		}
		if (isset($sent_data['consignee_seria_number'])) {
			$this->form_data->consignee_seria_number = $sent_data['consignee_seria_number'];
		} else {
			$this->form_data->consignee_seria_number = trim($this->form_data->consignee_seria . ' ' . $this->form_data->consignee_number);
		}							
		if (isset($sent_data['consignee_phone'])) {
			$this->form_data->consignee_phone = $sent_data['consignee_phone'];
		} else {
			if(isset($order->billing_address->address_telephone) && !empty($order->billing_address->address_telephone))
				$this->form_data->consignee_phone = $order->billing_address->address_telephone;
			if(isset($order->shipping_address->address_telephone) && !empty($order->shipping_address->address_telephone))
				$this->form_data->consignee_phone = $order->shipping_address->address_telephone;
			if(isset($order->billing_address->address_telephone2) && !empty($order->billing_address->address_telephone2))
				$this->form_data->consignee_phone = $order->billing_address->address_telephone2;
			if(isset($order->shipping_address->address_telephone2) && !empty($order->shipping_address->address_telephone2))
				$this->form_data->consignee_phone = $order->shipping_address->address_telephone2;				
		}					
		if (isset($sent_data['consignee_email'])) {
			$this->form_data->consignee_email = $sent_data['consignee_email'];
		} else {
			$this->form_data->consignee_email = $order->customer->email;
		}								
		if (isset($sent_data['departure_guid'])) {
			$this->form_data->departure_guid = $sent_data['departure_guid'];
		} else {
			$this->form_data->departure_guid = $config->shipping_params->departure_guid;
		}
		if (isset($sent_data['departure'])) {
			$this->form_data->departure = $sent_data['departure'];
		} else {
			$this->form_data->departure = $config->shipping_params->departure;
		}	
		if (isset($sent_data['departure_from_address'])) {
			$this->form_data->departure_from_address = $sent_data['departure_from_address'];
		} else {
			$this->form_data->departure_from_address = $config->shipping_params->departure_from_address;
		}
		if (isset($sent_data['departure_date'])) {
			$this->form_data->departure_date = $sent_data['departure_date'];
		} else {
			if($this->form_data->departure_from_address)
				$this->form_data->departure_date = date("Y-m-d", strtotime("+1 day"));
			else
				$this->form_data->departure_date = date("Y-m-d");
		}		
		if (isset($sent_data['departure_terminal_id'])) {
			$this->form_data->departure_terminal_id = $sent_data['departure_terminal_id'];
		} else {
			$this->form_data->departure_terminal_id = $config->shipping_params->departure_terminal_id;
		}	
		if (isset($sent_data['departure_street_guid'])) {
			$this->form_data->departure_street_guid = $sent_data['departure_street_guid'];
		} else {
			$this->form_data->departure_street_guid = $config->shipping_params->departure_street_guid;
		}
		if (isset($sent_data['departure_street'])) {
			$this->form_data->departure_street = $sent_data['departure_street'];
		} else {
			$this->form_data->departure_street = $config->shipping_params->departure_street;
		}
		if (isset($sent_data['departure_house_guid'])) {
			$this->form_data->departure_house_guid = $sent_data['departure_house_guid'];
		} else {
			$this->form_data->departure_house_guid = $config->shipping_params->departure_house_guid;
		}
		if (isset($sent_data['departure_house'])) {
			$this->form_data->departure_house = $sent_data['departure_house'];
		} else {
			$this->form_data->departure_house = $config->shipping_params->departure_house;
		}
		if (isset($sent_data['departure_housing'])) {
			$this->form_data->departure_housing = $sent_data['departure_housing'];
		} else {
			$this->form_data->departure_housing = $config->shipping_params->departure_housing;
		}
		if (isset($sent_data['departure_building'])) {
			$this->form_data->departure_building = $sent_data['departure_building'];
		} else {
			$this->form_data->departure_building = $config->shipping_params->departure_building;
		}
		if (isset($sent_data['departure_apartment'])) {
			$this->form_data->departure_apartment = $sent_data['departure_apartment'];
		} else {
			$this->form_data->departure_apartment = $config->shipping_params->departure_apartment;
		}
		if (isset($sent_data['departure_comment'])) {
			$this->form_data->departure_comment = $sent_data['departure_comment'];
		} else {
			$this->form_data->departure_comment = $config->shipping_params->departure_comment;
		}
		if (isset($sent_data['departure_prr'])) {
			$this->form_data->departure_prr = $sent_data['departure_prr'];
		} else {
			$this->form_data->departure_prr = $config->shipping_params->departure_prr;
		}
		if (isset($sent_data['departure_floor'])) {
			$this->form_data->departure_floor = $sent_data['departure_floor'];
		} else {
			$this->form_data->departure_floor = $config->shipping_params->departure_floor;
		}
		if (isset($sent_data['departure_lift'])) {
			$this->form_data->departure_lift = $sent_data['departure_lift'];
		} else {
			$this->form_data->departure_lift = $config->shipping_params->departure_lift;
		}
		if (isset($sent_data['destination_guid'])) {
			$this->form_data->destination_guid = $sent_data['destination_guid'];
		} else {
			$this->form_data->destination_guid = '';
			if(isset($order->billing_address->baikalsr_city) && !empty($order->billing_address->baikalsr_city))
				$this->form_data->destination_guid = $order->billing_address->baikalsr_city;
			if(isset($order->shipping_address->baikalsr_city) && !empty($order->shipping_address->baikalsr_city))
				$this->form_data->destination_guid = $order->shipping_address->baikalsr_city;
		}
		if (isset($sent_data['destination'])) {
			$this->form_data->destination = $sent_data['destination'];
		} else {
			$this->form_data->destination = '';
			if(isset($order->billing_address->city) && !empty($order->billing_address->address_city))
				$this->form_data->destination = $order->billing_address->address_city;
			if(isset($order->shipping_address->address_city) && !empty($order->shipping_address->address_city))
				$this->form_data->destination = $order->shipping_address->address_city;
		}
		if (isset($sent_data['destination_to_address'])) {
			$this->form_data->destination_to_address = $sent_data['destination_to_address'];
		} else {
			$this->form_data->destination_to_address = 0;
			$selected = null;
			foreach ($this->baikalsr_methods as $method) {
				if ($config->shipping_id.'-'.$method['key'] == $order->order_shipping_id)
					$selected = $method;
			}	
			if ($selected) {
				if ($selected['code']=='courier_shipping')
					$this->form_data->destination_to_address = 1;
				elseif ($selected['code']=='pickup_shipping')
					$this->form_data->destination_to_address = 0;
			}			
		}
		if (isset($sent_data['destination_date'])) {
			$this->form_data->destination_date = $sent_data['destination_date'];
		} else {
			if($this->form_data->departure_from_address && !$this->form_data->destination_to_address)
				$this->form_data->destination_date = date("Y-m-d", strtotime("+2 day"));
			elseif($this->form_data->departure_from_address && $this->form_data->destination_to_address)
				$this->form_data->destination_date = date("Y-m-d", strtotime("+3 day"));
			elseif(!$this->form_data->departure_from_address && $this->form_data->destination_to_address)
				$this->form_data->destination_date = date("Y-m-d", strtotime("+2 day"));
			else
				$this->form_data->destination_date = date("Y-m-d", strtotime("+1 day"));
		}			
		if (isset($sent_data['destination_terminal_id'])) {
			$this->form_data->destination_terminal_id = $sent_data['destination_terminal_id'];
		}
		if (isset($sent_data['destination_street_guid'])) {
			$this->form_data->destination_street_guid = $sent_data['destination_street_guid'];
		} else {
			$this->form_data->destination_street_guid = '';
		}
		if (isset($sent_data['destination_street'])) {
			$this->form_data->destination_street = $sent_data['destination_street'];
		} else {
			$this->form_data->destination_street = '';//$order->shipping_address->address_street . ', ' . $order->shipping_address->address_street2;
		}	
		if (isset($sent_data['destination_house_guid'])) {
			$this->form_data->destination_house_guid = $sent_data['destination_house_guid'];
		} else {
			$this->form_data->destination_house_guid = '';
		}
		if (isset($sent_data['destination_house'])) {
			$this->form_data->destination_house = $sent_data['destination_house'];
		} else {
			$this->form_data->destination_house = '';
		}
		if (isset($sent_data['destination_housing'])) {
			$this->form_data->destination_housing = $sent_data['destination_housing'];
		} else {
			$this->form_data->destination_housing = '';
		}
		if (isset($sent_data['destination_building'])) {
			$this->form_data->destination_building = $sent_data['destination_building'];
		} else {
			$this->form_data->destination_building = '';
		}
		if (isset($sent_data['destination_apartment'])) {
			$this->form_data->destination_apartment = $sent_data['destination_apartment'];
		} else {
			$this->form_data->destination_apartment = '';
		}	
		if (isset($sent_data['destination_comment'])) {
			$this->form_data->destination_comment = $sent_data['destination_comment'];
		} else {
			$this->form_data->destination_comment = '';
			if(isset($order->billing_address->address_state) && !empty($order->billing_address->address_state))
				$this->form_data->destination_comment = $order->billing_address->address_state;
			if(isset($order->shipping_address->address_state) && !empty($order->shipping_address->address_state))
				$this->form_data->destination_comment = $order->shipping_address->address_state;	
			$this->form_data->destination_comment .= ' ' . $this->form_data->destination . ' ';
			if($this->form_data->destination_to_address){
				if(isset($order->shipping_address->address_street) && !empty($order->shipping_address->address_street)){
					$this->form_data->destination_comment .= $order->shipping_address->address_street;
					if(isset($order->shipping_address->address_street2) && !empty($order->shipping_address->address_street2))
						$this->form_data->destination_comment .= ' ' . $order->shipping_address->address_street2;
				}elseif(isset($order->billing_address->address_street) && !empty($order->billing_address->address_street)){
					$this->form_data->destination_comment .=  $order->billing_address->address_street;
					if(isset($order->billing_address->address_street2) && !empty($order->billing_address->address_street2))
						$this->form_data->destination_comment .= ' ' . $order->billing_address->address_street2;
				}
			}else{
				if(isset($order->shipping_address->address_id) && !empty($order->shipping_address->address_id)){
					$response = $this->baikalsrTools::_getPreferredTerminal($order->shipping_address->address_id);
					if(empty($response)){
						if(isset($order->billing_address->address_id) && !empty($order->billing_address->address_id))
							$response = $this->baikalsrTools::_getPreferredTerminal($order->billing_address->address_id);
					}
				}
				if(!empty($response))
					$this->form_data->destination_comment .= '. ' . JText::_('PLG_HIKASHOPSHIPPING_TERMINAL_ADDRESS'). ': ' . $response->destination_terminal;
			}
			$this->form_data->destination_comment = trim($this->form_data->destination_comment);
		}	
		if (isset($sent_data['destination_prr'])) {
			$this->form_data->destination_prr = $sent_data['destination_prr'];
		} else {
			$this->form_data->destination_prr = 0;
		}	
		if (isset($sent_data['destination_floor'])) {
			$this->form_data->destination_floor = $sent_data['destination_floor'];
		} else {
			$this->form_data->destination_floor = '';
		}
		if (isset($sent_data['destination_lift'])) {
			$this->form_data->destination_lift = $sent_data['destination_lift'];
		} else {
			$this->form_data->destination_lift = 1;
		}	
		if (isset($sent_data['cargotype_id'])) {
			$this->form_data->cargotype_id = $sent_data['cargotype_id'];
		} else {
			$this->form_data->cargotype_id = $config->shipping_params->cargotype_id;
		}	
		if (isset($sent_data['packing_id'])) {
			$this->form_data->packing_id = $sent_data['packing_id'];
		} else {
			$this->form_data->packing_id = $config->shipping_params->packing_id;
		}			
		$prodDimensions = $this->_getDimensionsAndWeight($order, $config->shipping_params);
		if (isset($sent_data['volume'])) {
			$this->form_data->volume = $sent_data['volume'];
		} else {
			$this->form_data->volume = $prodDimensions['volume'];
		}	
		if (isset($sent_data['weight'])) {
			$this->form_data->weight = $sent_data['weight'];
		} else {
			$this->form_data->weight = $prodDimensions['weight'];
		}	
		if (isset($sent_data['length'])) {
			$this->form_data->length = $sent_data['length'];
		} else {
			$this->form_data->length = $prodDimensions['length'];
		}	
		if (isset($sent_data['width'])) {
			$this->form_data->width = $sent_data['width'];
		} else {
			$this->form_data->width = $prodDimensions['width'];
		}	
		if (isset($sent_data['height'])) {
			$this->form_data->height = $sent_data['height'];
		} else {
			$this->form_data->height = $prodDimensions['height'];
		}	
		if (isset($sent_data['estimated_cost'])) {
			$this->form_data->estimated_cost = $sent_data['estimated_cost'];
		} else {
			$this->form_data->estimated_cost = $order->order_subtotal;
		}
		
		return;
	}

	/**
	 * Этот метод будет вызван HikaShop при выборе способа доставки в процессе оформления заказа.
	 * Метод отображает список возможных вариантов доставки и запрашивает плагины доставки.
	 * В массиве $methods вы получите все опубликованные методы, найденные системой.
	 * Основываясь на ограничениях, установленных для вашего способа доставки, и на основании информации о заказе из $order, 
	 * необходимо будет решить, можно ли добавить ваш способод доставки в массив $usable_methods.
	 *
	 * @param object $element Метод доставки
	*/	
	function onShippingDisplay(&$order, &$dbrates, &$usable_rates, &$messages) {
		if(!hikashop_loadUser())
			return false;
		
		$local_usable_rates = array();
		$local_messages = array();
		$ret = parent::onShippingDisplay($order, $dbrates, $local_usable_rates, $local_messages);
		if($ret === false)
			return false;

		$currentShippingZone = null;
		$currentCurrencyId = null;
		$found = true;
		$usableWarehouses = array();
		$zoneClass = hikashop_get('class.zone');
		$zones = $zoneClass -> getOrderZones($order);
					
		foreach ($local_usable_rates as $k =>$rate) {
			if (empty($rate -> shipping_params -> methods)) {
				$messages['no_shipping_methods_configured'] = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_METHODS_CONF');
				continue;
			}

			if (empty($order -> shipping_address)) {
				$messages['no_shipping_address_found'] = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_SHIPPING_ADDRESS');
				continue;
			}

			$this -> shipping_currency_id = hikashop_getCurrency();
			$db = JFactory::getDBO();
			$query = 'SELECT currency_code FROM ' . hikashop_table('currency') . ' WHERE currency_id IN (' . $this -> shipping_currency_id . ')';
			$db -> setQuery($query);
			$this -> shipping_currency_code = $db -> loadResult();
			$cart = hikashop_get('class.cart');
			$null = null;
			$cart -> loadAddress($null, $order -> shipping_address -> address_id, 'object', 'shipping');
			$currency = hikashop_get('class.currency');

			$receivedMethods = $this -> _getShippingMethods($rate, $order);
			if (empty($receivedMethods)) {
				$messages['no_rates'] = JText::_('NO_SHIPPING_METHOD_FOUND');
				continue;
			}
			$doc = JFactory::getDocument();
			$doc->addScriptDeclaration('
			jQuery().ready(function ($) {						
				$(document).delegate("#address_city", "focus", function(e){
					var $el = $(this);
					if(!$el.attr("data-chars")){
						$el.attr("data-chars", "2");							
						var $hdn = $("input#baikalsr_city");
						$el.autocomplete({
							serviceUrl: "https://www.baikalsr.ru/json/api_fias_cities_v2.json",
							params: {}, 
							paramName: "text",
							type: "GET",
							dataType: "json",
							minChars: $el.attr("data-chars"),
							preserveInput: false,
							//triggerSelectOnValidInput: false,
							//deferRequestBy: 500, //500ms
							noCache: true,
							maxHeight: 400,
							width: "flex",
							zIndex: 9999,
							showNoSuggestionNotice: true,
							noSuggestionNotice: translate.no_data,
							transformResult: function(response) {
								return {
									suggestions: $.map(response, function(dataItem) {
										return { 
											value: (dataItem.parents) ? (dataItem.title + " (" + dataItem.parents + ")") : dataItem.title , 
											data: {
												id: dataItem.guid, 
												title: dataItem.title
											} 
										};
									})
								};
							},
							onSelect: function (suggestion) {
								$el.val(suggestion.data.title);
								$el.attr("data-title", suggestion.data.title);
								$hdn.val(suggestion.data.id);																		
							},
						}).blur(function () {
							if ($el.attr("data-title") != $el.val()) {
								$hdn.val("");
								$el.val("");
								$el.attr("data-title", "");	
							}
						});	
					}
				});
			});   
			');
			$i = 0;
			$local_usable_rates = array();
			foreach ($receivedMethods as $key => $method) {
				$local_usable_rates[$i] = clone($rate);
				$local_usable_rates[$i]->shipping_id .= '-' . $method['id'];
				$local_usable_rates[$i]->shipping_name .= ' (' . $method['name'] . ')';
				$local_usable_rates[$i]->shipping_price += $method['price'];
				if (isset($method['deliveryDate']) && !empty($method['deliveryDate'])) 
					$local_usable_rates[$i]->shipping_description .= JText::_('PLG_HIKASHOPSHIPPING_TERM_LBL') . ': ' . $method['deliveryDate'];
				if (isset($method['terminals']) && !empty($method['terminals'])){ 
					$destination_terminal_id = -1;
					if(isset($order->shipping_address->address_id) && !empty($order->shipping_address->address_id)){
						$response = $this->baikalsrTools::_getPreferredTerminal($order->shipping_address->address_id);
						if(empty($response)){
							if(isset($order->billing_address->address_id) && !empty($order->billing_address->address_id))
								$response = $this->baikalsrTools::_getPreferredTerminal($order->billing_address->address_id);
						}
					}
					if(!empty($response))
						$destination_terminal_id = $response->destination_terminal_id;
					
					$local_usable_rates[$i]->shipping_description .= '<div class="form-group">';
					$local_usable_rates[$i]->shipping_description .= '<span class="control-label" for="select-destination-terminal">' . JText::_('PLG_HIKASHOPSHIPPING_TERMINAL_ADDRESS') . ':&nbsp;</span>';
					$local_usable_rates[$i]->shipping_description .= '<select id="select-destination-terminal" class="hkc-sm-5">';
					foreach ($method['terminals'] as $terminalData) {
						if($terminalData['id'] == $destination_terminal_id)
							$local_usable_rates[$i]->shipping_description .= '<option value="'.$terminalData['id'].'" selected="selected">'.$terminalData['name'].'</option>';
						else
							$local_usable_rates[$i]->shipping_description .= '<option value="'.$terminalData['id'].'">'.$terminalData['name'].'</option>';
					}
					$local_usable_rates[$i]->shipping_description .= '</select>';
					$local_usable_rates[$i]->shipping_description .= '</div>';
                    $doc->addScriptDeclaration('
					jQuery().ready(function ($) {						
						$("#select-destination-terminal").on("change", function(){
							$.ajax({
								url: "/plugins/hikashopshipping/' . $this->name . '/iffi.php",
								data: {
									"destination_terminal_id":encodeURIComponent($(this).val()),
									"destination_terminal":encodeURIComponent($(this).find(\'option:selected\').text()),
									"address_id":encodeURIComponent(' . $order->shipping_address->address_id .'),
								},
								type: "POST",
								dataType: "json",
								success: function(){ 
									//console.log( "Всё ок" ); 
								}
							});
						});
					});   
					');
				}
				$i++;
			}
			foreach ($local_usable_rates as $i => $rate) {
				$usable_rates[$rate->shipping_id] = $rate;
			}
		}
	}

	function _getShippingMethods(&$rate, &$order) {		
		$methods = [];
		
		$total_price = $this->_getBasketPrice($order); 
		
		if (!empty($rate -> shipping_params -> methods)) {
			foreach ($rate->shipping_params->methods as $key => $value) {
				$selected = null;
				foreach ($this->baikalsr_methods as $method) {
					if ($method['code'] == $value)
						$selected = $method;
				}
				if ($selected) {
					//Установим ключ api для выполнения расчета
					if (isset($rate->shipping_params->test) && isset($rate->shipping_params->authKeyTest) && isset($rate->shipping_params->authKey)){
						if($rate->shipping_params->test){
							$this->baikalsrTools::defineAuthParams( $rate->shipping_params->authKeyTest, true );
						}else{
							$this->baikalsrTools::defineAuthParams( $rate->shipping_params->authKey, false );
						}				
					}
					$destination = $this->baikalsrTools::_getLocationWithGUIDFromAddress($order -> shipping_address);

					$total_dimensions = $this->_getDimensionsAndWeight($order, $rate -> shipping_params);
					$preorderData = array(
						'departure_guid' => $rate->shipping_params->departure_guid,
						'departure_from_address' => $rate->shipping_params->departure_from_address,
						'departure_street_guid' => $rate->shipping_params->departure_street_guid,
						'departure_house' => $rate->shipping_params->departure_house,
						'departure_housing' => $rate->shipping_params->departure_housing,
						'departure_building' => $rate->shipping_params->departure_building,
						'departure_prr' => $rate->shipping_params->departure_prr,
						'departure_floor' => $rate->shipping_params->departure_floor,
						'departure_lift' => $rate->shipping_params->departure_lift,
						'destination_guid' => $destination['guid'],
						//'destination_affiliate_guid' => $destination['affiliate']['guid'] ? $destination['affiliate']['guid'] : $destination['guid'],
						'destination_to_address' => true,
						'estimated_cost' => $total_price,
						'packing_id' => $rate->shipping_params->packing_id,
						'weight' => $total_dimensions['weight'],
						'length' => $total_dimensions['length'],
						'width' => $total_dimensions['width'],
						'height' => $total_dimensions['height'],
						'volume' => $total_dimensions['volume'],
					);
					$calcResult = $this->baikalsrTools::_calculate($preorderData);
					if (isset($calcResult["total"]) && !empty($calcResult["total"])){
						if(empty($rate->shipping_params->additional_cost)) $rate->shipping_params->additional_cost = 0.0;
						if(empty($rate->shipping_params->additional_cost_percent)) $rate->shipping_params->additional_cost_percent = 0.0;
				
						$destination_total = 0;
						if(isset($calcResult["destination"]["total"]) && !empty($calcResult["total"]))
							$destination_total = (float)$calcResult["destination"]["total"];
						if($selected['code'] == 'courier_shipping')
							$price = (float)$calcResult["total"] + $rate->shipping_params->additional_cost + (float)$calcResult["total"] * $rate->shipping_params->additional_cost_percent / 100;
						else
							$price = (float)$calcResult["total"] - $destination_total + $rate->shipping_params->additional_cost + ((float)$calcResult["total"] - $destination_total) * $rate->shipping_params->additional_cost_percent / 100;
						
						//Если подходит по условиям к бесплатной доставке
						if ($rate->shipping_params->free_shipment && $total_price >= $rate->shipping_params->free_shipment) {	
							$price = 0.0;
						}
						
						$methods[$selected['key']] = array(
							'id' => $selected['key'],
							'name' => JText::_($selected['name']),
							'price' => $price
						);
						//Разрешен ли вывод срока доставки в настройках метода доставки
						if($rate->shipping_params->show_term){
							if(isset($calcResult["transit"]["int"]) && isset($calcResult["transit"]["day"])){
								if($selected['code'] == 'courier_shipping'){
									$methods[$selected['key']]['deliveryDate'] = $calcResult["transit"]["int"] . ' ' . $calcResult["transit"]["day"];
								}elseif($selected['code'] == 'pickup_shipping'){
									if((int)$calcResult["transit"]["int"]>1)
										$calcResult["transit"]["int"] = (int)$calcResult["transit"]["int"] - 1;
									$methods[$selected['key']]['deliveryDate'] = $calcResult["transit"]["int"] . ' ' . $calcResult["transit"]["day"];
								}
							}
						}
						//Разрешен ли вывод терминалов в настройках метода доставки
						if($rate->shipping_params->show_terminals){
							if($selected['code'] == 'pickup_shipping'){
								$methods[$selected['key']]['terminals'] = [];
								$response = $this->baikalsrTools::_getAffiliateData($preorderData['destination_guid']);
								if (isset($response["terminals"]) && !empty($response["terminals"])) {
									foreach($response["terminals"] as $terminal) {
										if(isset($terminal["finish"]) && $terminal["finish"]){
											$terminal["address"] = $this->baikalsrTools::htmlspecialcharsBack($terminal["address"]);
											$methods[$selected['key']]['terminals'][] = array('id' => $terminal['id'], 'name' => $terminal['name']." (".$terminal["address"].")");
										}
									}
								}
							}
						}
					}
				}
			}
		}
		
		return $methods;
	}
	
	function _getBasketPrice($order){
        $totalPrice = 0;
        foreach($order->total->prices as $price){
			$totalPrice += $price->price_value_with_tax;
        }
        return $totalPrice;
    }
	
	function _getDimensionsAndWeight($order, $shipping_params) {
        $arDimensions = array();
		$defaultForItem = $shipping_params->default_for_item;
		$defLength = $this->baikalsrTools::convertLength($shipping_params->lengthD, $shipping_params->lwh_unit);
		$defWidth = $this->baikalsrTools::convertLength($shipping_params->widthD, $shipping_params->lwh_unit);
		$defHeight = $this->baikalsrTools::convertLength($shipping_params->heightD, $shipping_params->lwh_unit);
		$defWeight = $this->baikalsrTools::convertWeight($shipping_params->weightD, $shipping_params->weight_unit);
        $totalVolume1 = 0;
		$totalWeight1 = 0;
        $totalVolume2 = 0;
		$totalWeight2 = 0;
        $maxLength = 0;
        $maxWidth = 0;
        $maxHeight = 0;
        //max dimension
        foreach ($order->products as $product) {
			if(isset($order->order_id) && !empty($order->order_id)){
				if(!empty((float) $product->order_product_weight) && !empty($product->order_product_weight_unit)){
					$totalWeight1 += $product->order_product_quantity * $this->baikalsrTools::convertWeight($product->order_product_weight, $product->order_product_weight_unit);	
				}else{
					if($defaultForItem)
						$totalWeight2 += $product->order_product_quantity * $defWeight;
					else
						$totalWeight2 = $defWeight;
				}				
				if(!empty((float) $product->order_product_length) && !empty((float) $product->order_product_width) && !empty((float) $product->order_product_height) && !empty($product->order_product_dimension_unit)){
					$length = $this->baikalsrTools::convertLength($product->order_product_length, $product->order_product_dimension_unit);
					$width = $this->baikalsrTools::convertLength($product->order_product_width, $product->order_product_dimension_unit);
					$height = $this->baikalsrTools::convertLength($product->order_product_height, $product->order_product_dimension_unit);
					$totalVolume1 += $product->order_product_quantity * $length * $width * $height;
				}else{
					$length = $defLength;
					$width = $defWidth;
					$height = $defHeight;
					if($defaultForItem)
						$totalVolume2 += $product->order_product_quantity * $defLength * $defWidth * $defHeight;
					else
						$totalVolume2 = $defLength * $defWidth * $defHeight;
				}
			}else{
				if(!empty((float) $product->product_weight) && !empty($product->product_weight_unit)){
					$totalWeight1 += $product->cart_product_quantity * $this->baikalsrTools::convertWeight($product->product_weight, $product->product_weight_unit);	
				}else{
					if($defaultForItem)
						$totalWeight2 += $product->cart_product_quantity * $defWeight;
					else
						$totalWeight2 = $defWeight;
				}				
				if(!empty((float) $product->product_length) && !empty((float) $product->product_width) && !empty((float) $product->product_height) && !empty($product->product_dimension_unit)){
					$length = $this->baikalsrTools::convertLength($product->product_length, $product->product_dimension_unit);
					$width = $this->baikalsrTools::convertLength($product->product_width, $product->product_dimension_unit);
					$height = $this->baikalsrTools::convertLength($product->product_height, $product->product_dimension_unit);
					$totalVolume1 += $product->cart_product_quantity * $length * $width * $height;
				}else{
					$length = $defLength;
					$width = $defWidth;
					$height = $defHeight;
					if($defaultForItem)
						$totalVolume2 += $product->cart_product_quantity * $defLength * $defWidth * $defHeight;
					else
						$totalVolume2 = $defLength * $defWidth * $defHeight;
				}
			}
			if($length > $maxLength) $maxLength = $length;
			if($width > $maxWidth) $maxWidth = $width;
			if($height > $maxHeight) $maxHeight = $height;
        }
		$arDimensions['length']= $maxLength;
        $arDimensions['width'] = $maxWidth;
        $arDimensions['height'] = $maxHeight;
		$arDimensions['volume'] = $totalVolume1 + $totalVolume2;
		$arDimensions['weight'] = $totalWeight1 + $totalWeight2;
        return $arDimensions;
    }		

	public function onHikashopAfterDisplayView(&$view){

        $order_id = hikashop_getCID('order_id');

        if(empty($order_id))
            return false;
	
		if(!$this->init())
			return '';
		
        $orderClass = hikashop_get('class.order');
        $order = $orderClass->loadFullOrder($order_id);

        if($view->getName() == 'order' && $view->getLayout() == 'show'
            && $order->order_shipping_method == $this->name ){			
				echo $this->_getShippingInfoBody($order);
        }
    }
	
	public function _getShippingInfoBody($order)
	{
		$popupHelper = hikashop_get('helper.popup');
		$currencyClass = hikashop_get('class.currency');
				
		$db = JFactory::getDBO();
		$query = 'SELECT * FROM ' . hikashop_table('plg_baikalsr_shipping') . ' WHERE order_id = ' . $order->order_id. ' LIMIT 1';
		$db -> setQuery($query);
		$shipment = $db->loadObject();
		
		$html  = '<fieldset class="hika_field adminform" id="hikashop_' . $this->name . '_shipping_fieldset">';
		$html .= '<legend>' . JText::_('PLG_HIKASHOPSHIPPING_SHIPPING_INFO') . '</legend>';
		if(empty($shipment)){
			$html .= '<div class="hika_edit">';
			$html .= $popupHelper->display(
				'<i class="fa fa-plus"></i> ' . JText::_('HIKA_NEW'),
				'HIKASHOP_EDIT_CART_PRODUCT',
				hikashop_completeLink('plugins&plugin_type=shipping&task=edit&name='.$this->name.'&subtask=popupShow'.'&cid='.$order->order_id,true,true),
				'add_shipping_by_order_'.$order->order_id,
				800, 480, 'class="btn btn-primary"', '', 'link'
			); 
			$html .= '</div>';
		}
		$html .= '<table id="hikashop_' . $this->name . '_shipping_table" class="hika_listing hika_table table table-striped table-hover">';
		$html .= '	<thead>';
		$html .= '		<tr>';
		$html .= '			<th class="title">' . JText::_('PLG_HIKASHOPSHIPPING_REQUEST_ID') . '</th>';
		$html .= '			<th class="title">' . JText::_('DATE') . '</th>';
		$html .= '			<th class="title">' . JText::_('PLG_HIKASHOPSHIPPING_TRACKING_NUMBER') . '</th>';
		$html .= '			<th class="title">' . JText::_('PLG_HIKASHOPSHIPPING_SHIPPING_STATUS') . '</th>';
		$html .= '			<th class="title">' . JText::_('PLG_HIKASHOPSHIPPING_SHIPPING_COST') . '</th>';
		$html .= '			<th class="title">' . JText::_('PLG_HIKASHOPSHIPPING_ACTUAL_COST') . '</th>';
		$html .= '			<th class="title" colspan="2">' . JText::_('ACTIONS') . '</th>';
		$html .= '		</tr>';
		$html .= '	</thead>';
		if(!empty($shipment)){	
			$html .= '<tr>';
			$html .= '	<td>' . $shipment->request_id . '</td>';
			$html .= '	<td>' . $shipment->date_added . '</td>';
			$html .= '	<td>' . $shipment->tracking_number . '</td>';
			$html .= '	<td class="shipping_status">' . $shipment->status_name .  '</td>';
			$html .= '	<td class="shipping_cost">' . $currencyClass->format($shipment->shipment_cost, $order->order_currency_id) . '</td>';
			$html .= '	<td class="shipping_cost">';
			if(!empty($shipment->shipment_actual_cost) && (float)$shipment->shipment_actual_cost > 0)
				$html .= $currencyClass->format($shipment->shipment_actual_cost, $order->order_currency_id); 
			$html .= '	</td>';		
			$html .= '	<td style="text-align:center">';
			$html .= $popupHelper->display(
				'<i class="fas fa-eye"></i> ' . JText::_('HIKA_VIEW_MORE'),
				'HIKASHOP_EDIT_SHIPPING',
				hikashop_completeLink('plugins&plugin_type=shipping&task=edit&name='.$this->name.'&subtask=popupShow'.'&cid='.$order->order_id,true,true),
				'view_shipping_by_order_'.$order->order_id,
				800, 480, 'onclick="window.hikashop.openBox(this);return false;" class="btn btn-primary"', '', 'link'
			); 		
			$html .= '	</td>';	
			$html .= '	<td style="text-align:center">';	
			if(isset($shipment->request_id) && !empty($shipment->request_id)){
				$html .= '	<button type="button" class="btn btn-warning" onclick="return window.baikalsr.trackShipment(this, ' . $order->order_id .  ');"><i class="fas fa-sync"></i> ' . JText::_('PLG_HIKASHOPSHIPPING_BUTTON_SYNC') . '</button>';
			}
			$html .= '	</td>';
			$html .= '</tr>';
		}
		$html .= '</table>';	
		$html .= '</fieldset>';
		return $html;
	}
	
	protected function _validateForm($form_data) {

		$warning = new stdClass();
		if(!isset($form_data['consignor_contragent_id']) || $form_data['consignor_contragent_id'] == ''
			|| $form_data['consignor_contragent_id'] == '-1')
			$warning->consignor_contragent = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_CONTRAGENT');
		if(!isset($form_data['consignor_fio']) || $form_data['consignor_fio'] == '')
			$warning->consignor_fio = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_FIO');
		if(!isset($form_data['author_firstname']) || $form_data['author_firstname'] == '')
			$warning->consignor_fio = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_FIO');
		if(!isset($form_data['author_middlename']) || $form_data['author_middlename'] == '')
			$warning->consignor_fio = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_FIO');
		if(!isset($form_data['author_lastname']) || $form_data['author_lastname'] == '')
			$warning->consignor_fio = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_FIO');
		if(!isset($form_data['consignor_phone']) || $form_data['consignor_phone'] == '')
			$warning->consignor_phone = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_PHONE');
		elseif(strlen(preg_replace('![^0-9]+!', '', $form_data['consignor_phone'])) < 11)
			$warning->consignor_phone = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NOT_CORRECT_PHONE');
		if(!isset($form_data['consignor_email']) || $form_data['consignor_email'] == '')
			$warning->consignor_email = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_EMAIL');
		if(isset($form_data['consignee_physical_person']) && $form_data['consignee_physical_person'] != '1'){
			if(!isset($form_data['consignee_company']) || $form_data['consignee_company'] == '')
				$warning->consignee_company = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_COMPANY_NAME');
			if(!isset($form_data['consignee_legal_opf_id']) || $form_data['consignee_legal_opf_id'] == '' 
				|| $form_data['consignee_legal_opf_id'] == '-1')
				$warning->consignee_legal_opf = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_OPF');
			if(!isset($form_data['consignee_inn_kpp']) || $form_data['consignee_inn_kpp'] == ''){
				$warning->consignee_inn_kpp = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_INN_KPP');	
			}elseif(isset($form_data['consignee_legal_opf_id']) && !empty($form_data['consignee_legal_opf_id']) 
			&& $form_data['consignee_legal_opf_id'] != '9'){
				if( strlen(preg_replace('/[^0-9.]/', '', $form_data['consignee_inn']))<10)
					$warning->consignee_inn_kpp = JText::sprintf('PLG_HIKASHOPSHIPPING_ERROR_NOT_CORRECT_INN', '10');	
				if( strlen(preg_replace('/[^0-9.]/', '', $form_data['consignee_kpp']))<9)
					$warning->consignee_inn_kpp= JText::sprintf('PLG_HIKASHOPSHIPPING_ERROR_NOT_CORRECT_KPP', '9');					
			}elseif(isset($form_data['consignee_legal_opf_id']) && !empty($form_data['consignee_legal_opf_id']) 
			&& $form_data['consignee_legal_opf_id'] == '9' && strlen(preg_replace('/[^0-9.]/', '', $form_data['consignee_inn']))<12){
				$warning->consignee_inn_kpp = JText::sprintf('PLG_HIKASHOPSHIPPING_ERROR_NOT_CORRECT_INN', '12');	
			}
		}
		if(!isset($form_data['consignee_fio']) || $form_data['consignee_fio'] == '')
			$warning->consignee_fio = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_FIO');			
		if(isset($form_data['consignee_physical_person']) && $form_data['consignee_physical_person'] == '1'){
			if(!isset($form_data['consignee_no_document']) || $form_data['consignee_no_document'] != '1'){
				if(!isset($form_data['consignee_physical_document_id']) || $form_data['consignee_physical_document_id'] == '' 
					|| $form_data['consignee_physical_document_id'] == '-1')
					$warning->consignee_physical_document = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_DOCUMENT');	
				if(!isset($form_data['consignee_seria_number']) || $form_data['consignee_seria_number'] == '')
					$warning->consignee_seria_number = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_SERIA_NUMBER');		
			}
		}
		if(!isset($form_data['consignee_phone']) || $form_data['consignee_phone'] == '')
			$warning->consignee_phone = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_PHONE');
		elseif(strlen(preg_replace('![^0-9]+!', '', $form_data['consignee_phone'])) < 11)
			$warning->consignee_phone = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NOT_CORRECT_PHONE');
		if(!isset($form_data['consignee_email']) || $form_data['consignee_email'] == '')
			$warning->consignee_email = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_EMAIL');
		if(!isset($form_data['departure_guid']) || $form_data['departure_guid'] == '')
			$warning->departure = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_DEPARTURE');
		if(!isset($form_data['departure']) || $form_data['departure'] == '')
			$warning->departure = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_DEPARTURE');
		if(isset($form_data['departure_from_address']) && $form_data['departure_from_address'] != '1'){
			if(!isset($form_data['departure_terminal_id']) || $form_data['departure_terminal_id'] == '' 
				|| $form_data['departure_terminal_id'] == '-1')
				$warning->departure_terminal_id = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_TERMINAL');					
		}	
		if(isset($form_data['departure_from_address']) && $form_data['departure_from_address'] == '1'){
			if(!isset($form_data['departure_street_guid']) || $form_data['departure_street_guid'] == '')
				$warning->departure_street = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_STREET');		
			if(!isset($form_data['departure_street']) || $form_data['departure_street'] == '')
				$warning->departure_street = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_STREET');		
			//Можно вводить дом без классификатора
			//if(!isset($form_data['departure_house_guid']) || $form_data['departure_house_guid'] == '')
			//	$warning->departure_house = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_HOUSE');		
			if(!isset($form_data['departure_house']) || $form_data['departure_house'] == '')
				$warning->departure_house = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_HOUSE');	
			if(isset($form_data['departure_prr']) && $form_data['departure_prr'] == '1'){
				if(!isset($form_data['departure_floor']) || $form_data['departure_floor'] == '')
					$warning->departure_floor = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_FLOOR');	
			}
		}	
		if(!isset($form_data['destination_guid']) || $form_data['destination_guid'] == '')
			$warning->destination = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_DESTINATION');
		if(!isset($form_data['destination']) || $form_data['destination'] == '')
			$warning->destination = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_DESTINATION');
		if(isset($form_data['destination_to_address']) && $form_data['destination_to_address'] != '1'){
			if(!isset($form_data['destination_terminal_id']) || $form_data['destination_terminal_id'] == '' 
				|| $form_data['destination_terminal_id'] == '-1')
				$warning->destination_terminal_id = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_TERMINAL');					
		}			
		if(isset($form_data['destination_to_address']) && $form_data['destination_to_address'] == '1'){
			if(!isset($form_data['destination_street_guid']) || $form_data['destination_street_guid'] == '')
				$warning->destination_street = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_STREET');		
			if(!isset($form_data['destination_street']) || $form_data['destination_street'] == '')
				$warning->destination_street = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_STREET');		
			//Можно вводить дом без классификатора
			//if(!isset($form_data['destination_house_guid']) || $form_data['destination_house_guid'] == '')
			//	$warning->destination_house = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_HOUSE');		
			if(!isset($form_data['destination_house']) || $form_data['destination_house'] == '')
				$warning->destination_house = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_HOUSE');	
			if(isset($form_data['destination_prr']) && $form_data['destination_prr'] == '1'){
				if(!isset($form_data['destination_floor']) || $form_data['destination_floor'] == '')
					$warning->destination_floor = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_FLOOR');	
			}
		}	
		if(!isset($form_data['cargotype_id']) || $form_data['cargotype_id'] == '' 
			|| $form_data['cargotype_id'] == '-1')
			$warning->cargotype = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_CARGOTYPE');			
		if(!isset($form_data['volume']) || $form_data['volume'] == '' 
			|| $form_data['volume'] == '0')
			$warning->volume = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_VOLUME');		
		if(!isset($form_data['weight']) || $form_data['weight'] == '' 
			|| $form_data['weight'] == '0')
			$warning->weight = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_WEIGHT');					

		return $warning;
	}
	
	public function _updateShipmentOrder($shipment) {
        $values = array();
        $insert_data = array();
        $insert_data['id'] = (int)$shipment->id;
		$insert_data['order_id'] = (int)$shipment->order_id;
		$insert_data['shipmentmethod_id'] = (int)$shipment->shipmentmethod_id;
		$insert_data['request_id'] = "'" . $this->db->escape($shipment->request_id) . "'";
		$insert_data['order_number'] = "'" . $this->db->escape($shipment->order_number) . "'";		
        $insert_data['tracking_number'] = "'" . $this->db->escape($shipment->tracking_number) . "'";		
		$insert_data['shipment_data'] = "'" . $this->db->escape(json_encode($shipment->shipment_data)). "'";
        $insert_data['status_name'] = "'" . $this->db->escape($shipment->status_name) . "'";	
        $insert_data['shipment_cost'] = (float)$shipment->shipment_cost;
        $insert_data['shipment_actual_cost'] = (float)$shipment->shipment_actual_cost;
        $insert_data['shipment_add_cost'] = (float)$shipment->shipment_add_cost;
		if(isset($shipment->date_added) && !empty($shipment->date_added))
			$insert_data['date_added'] = "'" . $shipment->date_added . "'";
		else
			$insert_data['date_added'] = "NOW()";
        $insert_data['modified'] = 'NOW()';
		
        $values[] = "(" . implode(",", $insert_data) . ")";
		$db = JFactory::getDBO();
        $query = "INSERT INTO " . hikashop_table('plg_baikalsr_shipping') . " (`id`, `order_id`, `shipmentmethod_id`, `request_id`, `order_number`, `tracking_number`, `shipment_data`, `status_name`, `shipment_cost`, `shipment_actual_cost`, `shipment_add_cost`, `date_added`, `modified`) VALUES " . implode(',', $values) . " ON DUPLICATE KEY UPDATE `status_name`=VALUES(`status_name`), `shipment_actual_cost`= VALUES(`shipment_actual_cost`), `modified`=NOW()";
		$db->setQuery($query);
		$db->execute();
    }
	
	/**
	 * Этот метод вызывается при запросах (например, Ajax) к плагину из административной части сайта.
	 *
	 * @param string $type Тип плагина, например vmuserfield
	 * @param string $name Имя плагина
	 * @return $render mixed Null В качестве результата возвращает текст (HTML) или JSON.
	*/
	public function plgHikaOnSelfCallBE($action) {
		$action = hikaInput::get()->getCmd('subtask', '');
		switch ($action) {
			case 'popupShow':
				//getting order_id from url
				$order_id = hikashop_getCID('order_id');
				if(empty($order_id)){
					return NULL; 
				}
				$post_data = array();
				if (isset($_POST['params']) && !empty($_POST['params'])){
					$post_data = $_POST['params'];
				}
				$popup_data = $this->_getShippingPopupData($order_id, $post_data);
				$app = JFactory::getApplication();
				ob_start();
				include JPluginHelper::getLayoutPath('hikashopshipping', 'baikalsr', 'popup_shipping');	
				if(!$this->init())
					return '';				
				$html = ob_get_clean();
				$app->execute();
				echo $html;
				exit;	
			case 'getCities':
				hikashop_cleanBuffers();	
				$suggestions = [];		
				$query = hikaInput::get()->getString('query', '');  
				if(!empty($query)){
					$apiTypeTest = hikaInput::get()->getString('apiTypeTest', ''); 
					$apiKey = hikaInput::get()->getString('apiKey', ''); 
					$response = $this->baikalsrTools::getCitiesByName($query, "", $apiKey, $apiTypeTest);
					if (!empty($response) && is_array($response))
						foreach($response as $item){
							$suggestions[] = array(
								"value"=>!empty($item['parents']) ? $item['title'] . ' (' . $item['parents'] . ')' : $item['title'], 
								"data"=>[
									"id"=>$item["guid"],
									"title"=>$item['title']
								]
							);
						}
				}
				echo json_encode(["query"=>$query, "suggestions"=>$suggestions]);
				jExit();
				break;
			case 'getStreets':	
				hikashop_cleanBuffers();	
				$suggestions = [];				
				$query = hikaInput::get()->getString('query', '');  
				$guid = hikaInput::get()->getString('localityGuid', '');  
				if(!empty($guid) && !empty($query)){				
					$apiTypeTest = hikaInput::get()->getString('apiTypeTest', ''); 
					$apiKey = hikaInput::get()->getString('apiKey', ''); 
					$response = $this->baikalsrTools::getStreetsByName($guid, $query, $apiKey, $apiTypeTest);
					if (!empty($response) && is_array($response))
						foreach($response as $item){
							$suggestions[] = array(
								"value"=>!empty($item['parents']) ? $item['title'] . ' (' . $item['parents'] . ')' : $item['title'], 
								"data"=>[
									"id"=>$item["guid"],
									"title"=>$item['title']
								]
							);
						}	
				}
				echo json_encode(["query"=>$query, "suggestions"=>$suggestions]);
				jExit();
				break;
			case 'getHouses':	
				hikashop_cleanBuffers();	
				$suggestions = [];				
				$query = hikaInput::get()->getString('query', '');  
				$guid = hikaInput::get()->getString('streetGuid', ''); 
				if(!empty($guid) && !empty($query)){					
					$apiTypeTest = hikaInput::get()->getString('apiTypeTest', ''); 
					$apiKey = hikaInput::get()->getString('apiKey', ''); 
					$response = $this->baikalsrTools::getHousesByName($guid, $query, $apiKey, $apiTypeTest);
					if (!empty($response) && is_array($response))
						foreach($response as $item){
							$suggestions[] = array(
								"value"=>$item['formatted'], 
								"data"=>[
									"id"=>$item["houseguid"],
									"title"=>$item['housenum'],
									"buildnum"=>$item["buildnum"],
									"strucnum"=>$item['strucnum'],
								]
							);
						}
				}
				echo json_encode(["query"=>$query, "suggestions"=>$suggestions]);
				jExit();
				break;
			case 'getCargoTypes':	
				hikashop_cleanBuffers();	
				$json = array('elements' => []);			
				$apiTypeTest = hikaInput::get()->getString('apiTypeTest', ''); 
				$apiKey = hikaInput::get()->getString('apiKey', ''); 
				$response = $this->baikalsrTools::getCargoTypes($apiKey, $apiTypeTest);
				if(isset($response['error']) && !empty($response['error'])){
					$json['error'] = $response['error'];
				}else{
					foreach($response as $data) {
						if(isset($data['id']) && isset($data['name']))
							$json['elements'][] = array('id' => $data['id'], 'name' => $data['name']);
					}
				}
				echo json_encode($json);
				jExit();
				break;
			case 'getContragents':	
				hikashop_cleanBuffers();	
				$json = array('elements' => []);			
				$apiTypeTest = hikaInput::get()->getString('apiTypeTest', ''); 
				$apiKey = hikaInput::get()->getString('apiKey', ''); 
				$response = $this->baikalsrTools::getContragents($apiKey, $apiTypeTest);
				if(isset($response['error']) && !empty($response['error'])){
					$json['error'] = $response['error'];
				}else{
					foreach($response as $data) {
						if(isset($data['guid']) && isset($data['name']))
							$json['elements'][] = array('id' => $data['guid'], 'name' => $data['name']);
					}
				}
				echo json_encode($json);
				jExit();
				break;
			case 'getTerminals':	
				hikashop_cleanBuffers();	
				$json = array('elements' => []);
				$guid =  hikaInput::get()->getString('localityGuid', ''); 
				if(!empty($guid)){
					$terminal_type = hikaInput::get()->getString('terminalType', 'start'); 											
					$apiTypeTest = hikaInput::get()->getString('apiTypeTest', ''); 
					$apiKey = hikaInput::get()->getString('apiKey', ''); 
					$response = $this->baikalsrTools::getAffiliateData($guid, $apiKey, $apiTypeTest);

					if(isset($response['error']) && !empty($response['error'])){
						$json['error'] = $response['error'];
					}else{
						if (isset($response["terminals"]) && !empty($response["terminals"])) {
							foreach($response["terminals"] as $terminal) {
								if(isset($terminal[$terminal_type]) && $terminal[$terminal_type]){
									$terminal["address"] = $this->baikalsrTools::htmlspecialcharsBack($terminal["address"]);
									$json['elements'][] = array('id' => $terminal['id'], 'name' => $terminal['name']." (".$terminal["address"].")");
								}
							}
						}
					}
				}
				echo json_encode($json);
				jExit();
				break;
			case 'submitCalcForm':
				hikashop_cleanBuffers();	
				$json = array();
				//getting order_id from url
				$order_id = hikashop_getCID('order_id');
				$orderClass = hikashop_get('class.order');
				$order = $orderClass->loadFullOrder($order_id);
				if(empty($order)){
					$json['error'] = JText::_('PLG_HIKASHOPSHIPPING_NO_ENTRY_FOUND');
				}		
				//get shipment config
				$shippingClass = hikashop_get('class.shipping');
				$config = $shippingClass->get($order->order_shipping_id);
				//Установим ключ api для выполнения расчета
				if (isset($config->shipping_params->test) && isset($config->shipping_params->authKeyTest) && isset($config->shipping_params->authKey)){
					if($config->shipping_params->test){
						$this->baikalsrTools::defineAuthParams( $config->shipping_params->authKeyTest, true );
					}else{
						$this->baikalsrTools::defineAuthParams( $config->shipping_params->authKey, false );
					}				
				}else{
					$json['error'] = JText::_('PLG_HIKASHOPSHIPPING_ERROR_NO_METHODS_CONF');
				}
				if(!isset($json['error'])){
					if (isset($_POST['params']) && !empty($_POST['params'])){									
						$json = $this->baikalsrTools::calculate($_POST['params']);	
					}else{
						$json['error'] = JText::_('PLG_HIKASHOPSHIPPING_ERROR_REQUEST');
					}
				}
				echo json_encode($json);
				jExit();
				break;
			case 'tracking':
				hikashop_cleanBuffers();
				$json = array();
				$order_id = hikashop_getCID('order_id');
				$orderClass = hikashop_get('class.order');
				$order = $orderClass->loadFullOrder($order_id);
				if(empty($order)){
					$json['error'] = JText::_('PLG_HIKASHOPSHIPPING_ERROR_REQUEST');
				}			
				//get shipment order by order_id
				$db = JFactory::getDBO();
				$query = 'SELECT * FROM ' . hikashop_table('plg_baikalsr_shipping') . ' WHERE order_id = ' . $order_id. ' LIMIT 1';
				$db -> setQuery($query);
				$shipment = $db->loadObject();
				if (empty($shipment)){
					$json['error'] = JText::_('PLG_HIKASHOPSHIPPING_ERROR_REQUEST');
				}
				
				if(!isset($json['error'])){
					$shippingClass = hikashop_get('class.shipping');
					$config = $shippingClass->get($order->order_shipping_id);
					//Установим ключ api для выполнения расчета
					if (isset($config->shipping_params->test) && isset($config->shipping_params->authKeyTest) && isset($config->shipping_params->authKey)){
						if($config->shipping_params->test){
							$this->baikalsrTools::defineAuthParams( $config->shipping_params->authKeyTest, true );
						}else{
							$this->baikalsrTools::defineAuthParams( $config->shipping_params->authKey, false );
						}				
					}
					$baikalResponse = $this->baikalsrTools::getOrderData($shipment->request_id);
					if(isset($baikalResponse["status"]["name"]) && !empty($baikalResponse["status"]["name"])){
						$shipment->status_name = $baikalResponse["status"]["name"];
					}else{
						$json['error'] = JText::_('PLG_HIKASHOPSHIPPING_ERROR_API_NO_RESPONSE');
					}
					if (isset($baikalResponse["total"]["sum"]) && !empty($baikalResponse["total"]["sum"])) {
						$shipment->shipment_actual_cost = (float) $baikalResponse["total"]["sum"];
					}else{
						$json['error'] = JText::_('PLG_HIKASHOPSHIPPING_ERROR_API_NO_RESPONSE');
					}		
					$this->_updateShipmentOrder($shipment);
					$json['html'] = $this->_getShippingInfoBody($order);					
				}
				
				echo json_encode($json);
				jExit();
				break;	
			default:
				//$this->baikalsrError(JText::_('PLG_HIKASHOPSHIPPING_ERROR_API_NO_RESPONSE'));
				return;
		}
	}
}
