<?php
namespace Baikalsr\Lib;

class Api
{	
    const DOMAIN_TEST = "https://test-api.baikalsr.ru/";
    const DOMAIN_PROD = "https://api.baikalsr.ru/";
	
	static $APIKEY;
	static $TEST;
	
	static $shipping_params;
		
	public function __construct()
	{
		if (!function_exists('curl_init'))
		{
			throw new ApiException('cURL library is not loaded.');
		}
	}
	
	public static function defineAuthParams($apiKey = null, $apiTypeTest = null)
	{
		if(!empty($apiKey)){
			self::$APIKEY = $apiKey;
			self::$TEST = $apiTypeTest;
		}
    }
	
    private static function apiGetRequest($apiUrl, $apiKey = null, $apiTypeTest = null)
    {
		self::defineAuthParams($apiKey, $apiTypeTest);
		if (self::$TEST) {
			$apiUrl = self::DOMAIN_TEST.$apiUrl;
		}else{
			$apiUrl = self::DOMAIN_PROD.$apiUrl;			
		}
        if (self::$APIKEY) {
            if ($objCurl = curl_init()) {
                curl_setopt_array($objCurl, array(
                    CURLOPT_URL => $apiUrl,
                    CURLOPT_HTTPAUTH => CURLAUTH_BASIC,
                    CURLOPT_USERPWD => self::$APIKEY.':',
                    CURLOPT_RETURNTRANSFER => true,
                    CURLOPT_SSL_VERIFYPEER => false,
                    CURLOPT_TIMEOUT => 60,
                    CURLOPT_HTTPHEADER => array(
                        'Content-Type: application/json'
                    )
                ));
                $result = curl_exec($objCurl);
                if ($result === false) {
                    $error = 'CURL error: ' . curl_error($objCurl);
                    return array("Errors" => [$error]);
                }
                curl_close($objCurl);
            }
            return json_decode($result, true);
        }
        return array();
    }

    private static function apiPostRequest($apiUrl, $params = [], $apiKey = null, $apiTypeTest = null)
    {
      	self::defineAuthParams($apiKey, $apiTypeTest);
		if (self::$TEST) {
			$apiUrl = self::DOMAIN_TEST.$apiUrl;
		}else{
			$apiUrl = self::DOMAIN_PROD.$apiUrl;			
		}
        if (self::$APIKEY) {
            if ($objCurl = curl_init()) {
                curl_setopt_array($objCurl, array(
                    CURLOPT_URL => $apiUrl,
                    CURLOPT_HTTPAUTH => CURLAUTH_BASIC,
                    CURLOPT_USERPWD => self::$APIKEY.':',
                    CURLOPT_RETURNTRANSFER => true,
                    CURLOPT_POST => true,
                    CURLOPT_POSTFIELDS => json_encode($params),
                    CURLOPT_SSL_VERIFYPEER => false,
                    CURLOPT_TIMEOUT => 60,
                    CURLOPT_HTTPHEADER => array(
                        'Content-Type: application/json'
                    )
                ));
                $result = curl_exec($objCurl);
                if ($result === false) {
                    $error = 'CURL error: ' . curl_error($objCurl);
                    return array("Errors" => [$error]);
                }
                curl_close($objCurl);
            }
            return json_decode($result, true);
        }
        return array();
    }
	
    public static function getCitiesByName($city, $region = "", $apiKey = null, $apiTypeTest = null)
    {
        $allCitiesData = [];
        $apiUrl = "v2/fias/cities?text=";
        if ($city) {
            $apiUrl .= $city;
            if ($region) $apiUrl .= " ".$region;

            $allCitiesData = self::apiGetRequest($apiUrl, $apiKey, $apiTypeTest);
        }

        return $allCitiesData;
    }

    public static function getCityByName($city, $region = "", $apiKey = null, $apiTypeTest = null)
    {
        $cityData = [];
        $apiUrl = "v2/fias/cities?text=";
        if ($city) {
            $apiUrl .= $city;
            if ($region) $apiUrl .= " ".$region;
            $allCityData = self::apiGetRequest($apiUrl, $apiKey, $apiTypeTest);
            if(!empty($allCityData) && isset($allCityData[0]))
				if ($allCityData[0]) {
					$cityData = $allCityData[0];
				}
        }

        return $cityData;
    }

    public static function getCityByGuid($guid, $apiKey = null, $apiTypeTest = null)
    {
        $cityData = [];
        $apiUrl = "v2/fias/cities?guid=";
        if ($guid) {
            $apiUrl .= $guid;

            $response = self::apiGetRequest($apiUrl, $apiKey, $apiTypeTest);
            if (isset($response[0]) && !empty($response[0])) {
                $cityData = $response[0];
            }
        }

        return $cityData;
    }

    public static function getStreetsByName($cityGuid, $street = "", $apiKey = null, $apiTypeTest = null)
    {
        $allStreetsData = [];
        $apiUrl = "v2/fias/streets?guid=".$cityGuid."&text=";
        if ($street) {
            $apiUrl .= $street;

            $allStreetsData = self::apiGetRequest($apiUrl, $apiKey, $apiTypeTest);
        }

        return $allStreetsData;
    }
	
	public static function getStreetByGuid($guid, $apiKey = null, $apiTypeTest = null)
    {
        $data = [];
        $apiUrl = "v2/fias/streets?streetguid=";
        if ($guid) {
            $apiUrl .= $guid;

            $response = self::apiGetRequest($apiUrl, $apiKey, $apiTypeTest);
            if (isset($response[0]) && !empty($response[0])) {
                $data = $response[0];
            }
        }

        return $data;
    }
	
    public static function getHousesByName($streetGuid, $house = "", $apiKey = null, $apiTypeTest = null)
    {
        $allHousesData = [];
        $apiUrl = "v2/fias/objects?guid=".$streetGuid."&h=";
        if ($house) {
            $apiUrl .= $house;

            $allHousesData = self::apiGetRequest($apiUrl, $apiKey, $apiTypeTest);
        }

        return $allHousesData;
    }
	
    public static function getAffiliateData($guid, $apiKey = null, $apiTypeTest = null)
    {
        $affiliateData = [];
        $apiUrl = "v1/affiliate/";
        if ($guid) {
            $apiUrl .= $guid;

            $affiliateData = self::apiGetRequest($apiUrl, $apiKey, $apiTypeTest);
        }

        return $affiliateData;
    }
	
	public static function getCargoTypes($apiKey = null, $apiTypeTest = null)
    {
        $cargoTypes = [];
        $apiUrl = "v2/directory/cargotype";
        $cargoTypes = self::apiGetRequest($apiUrl, $apiKey, $apiTypeTest);

        return $cargoTypes;
    }	

	public static function getServices($guid, $apiKey = null, $apiTypeTest = null)
    {
        $services = [];
        if ($guid) {
            $apiUrl = "v2/directory/services/".$guid;
            $services = self::apiGetRequest($apiUrl, $apiKey, $apiTypeTest);
        }

        return $services;
    }

	public static function getPartnerTypes($apiKey = null, $apiTypeTest = null)
    {
        $partnerTypes = [];
        $apiUrl = "v2/directory/partnertype";
        $partnerTypes = self::apiGetRequest($apiUrl, $apiKey, $apiTypeTest);

        return $partnerTypes;
    }	
	
	public static function getContragents($apiKey = null, $apiTypeTest = null)
    {
        $contragents = [];
        $apiUrl = "v2/contragent/list";
        $contragents = self::apiPostRequest($apiUrl, [], $apiKey, $apiTypeTest);

        return $contragents;
    }
	
	public static function getDateOptions($guid)
    {
        $intervals = [];
        if ($guid) {
            $apiUrl = "v1/calculator/dateoptions";
            $params = [
                "guid" => $guid
            ];
            $intervals = self::apiPostRequest($apiUrl, $params);
        }

        return $intervals;
    }
	
	public static function getIntervals($guid, $apiKey = null, $apiTypeTest = null)
    {
        $intervals = [];
        if ($guid) {
            $apiUrl = "v2/directory/deliveryintervals/".$guid;
            $intervals = self::apiGetRequest($apiUrl, $apiKey, $apiTypeTest);
        }

        return $intervals;
    }
	
	public static function calculate($preorderData)
    {
        $apiUrl = "v2/calculator";

        $result = [];
		
        $params = [
            "Departure" => [
                "CityGuid" => $preorderData["departure_guid"] ? $preorderData["departure_guid"] : "",
            ],
            "Destination" => [
                "CityGuid" => $preorderData["destination_guid"], 
				//"CityGuid" => $preorderData["destination_to_address"] ? $preorderData["destination_guid"] : $preorderData["destination_affiliate_guid"],
            ]
		];
		if ($preorderData["departure_from_address"]) {
            $params["Departure"]["PickUp"] = [
                "Street" => isset($preorderData["departure_street_guid"]) ? $preorderData["departure_street_guid"] : "",
                "House" => isset($preorderData["departure_house"]) ? $preorderData["departure_house"] : "",
                "Housing" => isset($preorderData["departure_housing"]) ? $preorderData["departure_housing"] : "",
                "Building" => isset($preorderData["departure_building"]) ? $preorderData["departure_building"] : "",
                "Date" => date("Y-m-d", strtotime("+1 day"))."T".date("00:00:00"),
                "TimeFrom" => "09:00",
                "TimeTo" => "18:00",
               // "Services" => [3232]
            ];
            if (isset($preorderData["departure_prr"]) && !empty($preorderData["departure_prr"])) {
                $params["Departure"]["PickUp"]["Prr"] = [
                    "floor" => $preorderData["departure_floor"],
                    "lift" => $preorderData["departure_lift"] ? 1 : 0
                ];
            }
        }
		if ($preorderData["destination_to_address"]) {
            $params["Destination"]["Delivery"] = [
                "Street" => isset($preorderData["destination_street_guid"]) ? $preorderData["destination_street_guid"] : "",
                "House" => isset($preorderData["destination_house"]) ? $preorderData["destination_house"] : "",
                "Housing" => isset($preorderData["destination_housing"]) ? $preorderData["destination_housing"] : "",
                "Building" => isset($preorderData["destination_building"]) ? $preorderData["destination_building"] : "",
                "TimeFrom" => "09:00",
                "TimeTo" => "18:00"
            ];
            if (isset($preorderData["destination_prr"]) && !empty($preorderData["destination_prr"])) {
                $params["Destination"]["Delivery"]["Prr"] = [
                    "floor" => $preorderData["destination_floor"],
                    "lift" => $preorderData["destination_lift"] ? 1 : 0
                ];
            }

        }
		
		if(!isset($preorderData["volume"]) || empty($preorderData["volume"]))
			$preorderData["volume"] = $preorderData["length"] * $preorderData["width"] * $preorderData["height"];
		
		$params["Cargo"] = [
			"SummaryCargo" => [
				"Length" => $preorderData["length"],
				"Width" => $preorderData["width"],
				"Height" => $preorderData["height"],
				"Volume" => $preorderData["volume"],
				"Weight" => $preorderData["weight"],
				"Units" => 1,
				//"Oversized" => 1,
				"EstimatedCost" => $preorderData["estimated_cost"] ? $preorderData["estimated_cost"] : 0
			]
        ];
		
		if($preorderData["packing_id"] > 0){
			$fields[] = array("id"=>$preorderData["packing_id"], "value"=>1);
			$params["Cargo"]["SummaryCargo"]["servicesObject"][] = array("id"=>$preorderData["packing_id"], "fields"=>$fields);
		}

        $result = self::apiPostRequest($apiUrl, $params);

        //file_put_contents($_SERVER["DOCUMENT_ROOT"]."/ifscr/calc_log3.txt", $prF, FILE_APPEND);

        return $result;
    }
	
	public static function order($orderData)
    {
        $apiUrl = "v2/order";

        $result = [];

        //$volume = $orderData["gabarites"]["LENGTH"] * $orderData["gabarites"]["WIDTH"] * $orderData["gabarites"]["HEIGHT"];
        //list($opfId, $legalForm) = explode("_", $orderData["opf"]);
        $params = [
            "Author" => [
                "FirstName" => $orderData["author_firstname"],
                "MiddleName" => $orderData["author_middlename"],
                "LastName" => $orderData["author_lastname"],
                "Email" => $orderData["consignor_email"],
                "Phone" => $orderData["consignor_phone"],
                "Comment" => trim("ID заказа на сайте ".$orderData["order_id"])
            ],
            "Departure" => [
                "CityGuid" => $orderData["departure_guid"],
                //"PickUp" => [
                //    "StreetGuid" => "177e037d-a06d-4f8a-9333-fd534dd34c5d",
                //    "house" => "23",
                //    "housing" => "",
                //    "building" => "",
                //    "apartment" => "11",
                //    "comment" => "звонить в звонок",
                //    "Date" => "2020-12-31T00:00:00",
                //    "TimeFrom" => "10:00",
                //    "TimeTo" => "18:00",
                //    "Services" => [
                //        [
                //            "id" => 1,
                //            "PayerID" => "Consignor",
                //            "FormOfPayment" => 2
                //        ]
                //    ],
                //    "PayerID" => "Consignor",
                //    "FormOfPayment" => 2
                //]
            ],
            "OrdersList" => [
                [
                    "Destination" => [
                        //"Consignee" => [
                        //    "guid" => "",
                        //    "LegalForm" => $legalForm,
                        //    "Type" => $opfId,
                        //    "Name" => $orderData["company"],
                        //    "FirstField" => $orderData["inn"],
                        //    "SecondField" => $orderData["kpp"],
                        //    "Phone" => $orderData["phone"],
                        //   "Email" => $orderData["email"],
                        //    "ContactPerson" => $orderData["name"]
                        //],
                        "CityGuid" => $orderData["destination_guid"],
                    ],
                    "SummaryCargo" => [
                        "Length" => $orderData["length"],
                        "Width" => $orderData["width"],
                        "Height" => $orderData["height"],
                        "Volume" => $orderData["volume"],
                        "Weight" => $orderData["weight"],
                        "Units" => 1,
                        "Type" => $orderData["cargotype_id"],
                        "Insurance" => [
                            "EstimatedCost" => $orderData["estimated_cost"],
                            "PayerID" => "Consignor",
                            "FormOfPayment" => 2
                        ]
                    ],
                    "PayerID" => "Consignor",
                    "FormOfPayment" => 2
                ]
            ],
            "Payers" => [
                //[
                //    "PayerID" => "Payer1",
                //    "GUID" => "4f86a22c-4758-4859-a227-a79c9ef500ff",
                //    "LegalForm" => 1,
                //    "Type" => 234,
                //    "Name" => "ООО Юпитер",
                //    "FirstField" => "1231231230",
                //    "SecondField" => "123123001",
                //    "Phone" => "+79513579513",
                //    "Email" => "payer@mail.ru",
                //    "ContactPerson" => "Эдуард"
                //]
            ]
        ];
        if ($orderData["consignor_contragent_id"]) {
            $params["Departure"]["Consignor"] = [
                "guid" => $orderData["consignor_contragent_id"],
                "Phone" => $orderData["consignor_phone"],
                "Email" => $orderData["consignor_email"],
                "ContactPerson" => $orderData["consignor_fio"]
            ];	
        }else {
			$params["Departure"]["Consignor"] = [
				"guid" => "",
				"LegalForm" => 1,
				"Type" => 5,
				"Name" => "Вектор",
				"FirstField" => "1745567896",
				"SecondField" => "170100100",
                "Phone" => $orderData["consignor_phone"],
				"Email" => $orderData["consignor_email"],
                "ContactPerson" => $orderData["consignor_fio"]
            ];
            //list($opfId, $legalForm) = explode("_", $orderData["sender_opf"]);
            //$params["Departure"]["Consignor"] = [
            //    "guid" => "",
            //    "LegalForm" => $legalForm,
            //    "Type" => $opfId,
            //    "Name" => $orderData["sender_company"],
            //    "FirstField" => $orderData["sender_inn"],
            //    "SecondField" => $orderData["sender_kpp"],
            //    "Phone" => $orderData["sender_phone"],
            //    "Email" => $orderData["sender_email"],
            //    "ContactPerson" => $orderData["sender_name"]
            //];
        }
        if ($orderData["departure_from_address"]) {
            $params["Departure"]["PickUp"] = [
                "StreetGuid" => $orderData["departure_street_guid"],
                "house" => $orderData["departure_house"],
                "housing" => $orderData["departure_housing"],
                "building" => $orderData["departure_building"],
                "apartment" => $orderData["departure_apartment"],
                "comment" => $orderData["departure_comment"],
                "Date" => $orderData["departure_date"]."T00:00:00", //date("Y-m-d", strtotime("+1 day"))."T00:00:00",//"2020-12-31T00:00:00",  
                //"TimeFrom" => "10:00",
                //"TimeTo" => "18:00",
                //"Services" => [
                //    [
                //        "id" => 1,
                //        "PayerID" => "Consignor",
                //        "FormOfPayment" => 2
                //    ]
                //],
                "PayerID" => "Consignor",
                "FormOfPayment" => 2
            ];
            if (isset($orderData["departure_prr"]) && !empty($orderData["departure_prr"])) {
                $params["Departure"]["PickUp"]["prr"] = [
                    "floor" => $orderData["departure_floor"],
                    "lift" => isset($orderData["departure_lift"]) ? true : false
                ];
            }
        } else {
            $params["Departure"]["Terminal"] = $orderData["departure_terminal_id"];
        }
        if ($orderData["consignee_physical_person"]) {
            $params["OrdersList"][0]["Destination"]["Consignee"] = [
                "guid" => "",
                "LegalForm" => 2,
                "Type" => $orderData["consignee_physical_document_id"],
                "Name" => $orderData["consignee_fio"],
                "FirstField" => $orderData["consignee_seria"],
                "SecondField" => $orderData["consignee_number"],
                "Phone" => $orderData["consignee_phone"],
                "Email" => $orderData["consignee_email"]
            ];
        } else {
			$legalForm = 1;
			if($orderData["consignee_legal_opf_id"] == 9)
				$legalForm = 4;
			elseif($orderData["consignee_legal_opf_id"] == 2309)
				$legalForm = 3;
            $params["OrdersList"][0]["Destination"]["Consignee"] = [
                "guid" => "",
                "LegalForm" => $legalForm,
                "Type" => $orderData["consignee_legal_opf_id"],
                "Name" => $orderData["consignee_company"],
                "FirstField" => $orderData["consignee_inn"],
                "SecondField" => $orderData["consignee_kpp"],
                "Phone" => $orderData["consignee_phone"],
                "Email" => $orderData["consignee_email"],
                "ContactPerson" => $orderData["consignee_fio"]
            ];
        }
        if ($orderData["destination_to_address"]) {
            $params["OrdersList"][0]["Destination"]["Delivery"] = [
                "StreetGuid" => $orderData["destination_street_guid"],
                "House" => $orderData["destination_house"],
                "Housing" => $orderData["destination_housing"],
                "Building" => $orderData["destination_building"],
                "Apartment" => $orderData["destination_apartment"],
                "CustomAddress" => "",
                "Comment" => $orderData["destination_comment"],
                //"TimeFrom" => "10:00",
                //"TimeTo" => "18:00",
                //"Services" => [
                //    [
                //        "id" => 1,
                //        "PayerID" => "Consignor",
                //        "FormOfPayment" => 2
                //    ]
                //],
                "PayerID" => "Consignor",
                "FormOfPayment" => 2
            ];
			if (isset($orderData["destination_prr"]) && !empty($orderData["destination_prr"])) {
                $params["OrdersList"][0]["Destination"]["Delivery"]["prr"] = [
                    "floor" => $orderData["destination_floor"],
                    "lift" => isset($orderData["destination_lift"])? true : false
                ];
            }
        } else {
            $params["OrdersList"][0]["Destination"]["Terminal"] = $orderData["destination_terminal_id"];
        }
        if ($orderData["packing_id"]) {
            //foreach($orderData["PACKING"] as $pack) {
				$fields = [];
				$fields[] = array("id"=>$orderData["packing_id"], "value"=>1);
                $params["OrdersList"][0]["SummaryCargo"]["Services"][] = [
                    "id" => $orderData["packing_id"],
					"fields" => $fields,
                    "PayerID" => "Consignor",
                    "FormOfPayment" => 2
                ];
            //}
        }

        $result = self::apiPostRequest($apiUrl, $params);

        return $result;
    }
	
	public static function getOrderData($orderId)
    {
        $orderData = [];
        if ($orderId) {
            $apiUrl = "v2/order/detail";
            $orderData = self::apiPostRequest($apiUrl, ["id" => $orderId]);
        }

        return $orderData;
    }
	
	public static function convertLength($length, $length_code){
		$length = (float) $length;
        $length_code = strtolower($length_code);
		if ($length_code == 'дюйм' || $length_code == 'in') {
			$length = $length/39;
		}else if ($length_code == 'мм' || $length_code == 'mm') {
			$length = $length/1000;
		}else if ($length_code == 'см' || $length_code == 'cm') {
			$length = $length/100;
		}
		
        return $length;
    }	
	
	public static function convertWeight($weight, $weight_code){
		$weight = (float) $weight;
        $weight_code = strtolower($weight_code);

		if ($weight_code == 'г' || $weight_code == 'g') {
			$weight = $weight/1000;
		}

		if ($weight_code == 'lb') {
			$weight = $weight/2.20460000;
		}

		if ($weight_code == 'oz') {
			$weight = $weight/35.274;
		}
		
        return $weight;
    }	
	
	public static function cleanRequest($value)
    {
        if (is_array($value)) {
            foreach($value as $key=>&$sub_value) {
                if ($key == "table") {

                } else {
                    $sub_value = self::cleanRequest($sub_value);
                }
            }
            unset($sub_value);
        } else {
            $value = trim($value);
            $value = stripslashes($value);
            $value = strip_tags($value);
            $value = htmlspecialchars($value, ENT_NOQUOTES);
        }
        return $value;
    }
	
}